//
//  ViewController.swift
//  SwiftExample
//

import UIKit
import SQLite

internal let SQLITE_STATIC = unsafeBitCast(0, to: sqlite3_destructor_type.self)
internal let SQLITE_TRANSIENT = unsafeBitCast(-1, to: sqlite3_destructor_type.self)

class ViewController: UIViewController, UITableViewDataSource, UITableViewDelegate {

    var db: Connection!
    let tasks = Table("tasks")
    let id = Expression<Int64>("id")
    let name = Expression<String>("name")
    let done = Expression<Int>("done")

    var taskList = [Task]()

    @IBOutlet weak var tableViewTasks: UITableView!
    @IBOutlet weak var textFieldName: UITextField!

    @IBAction func buttonSave(_ sender: UIButton) {
        var taskname = textFieldName.text!

        taskname = taskname.trimmingCharacters(in: .whitespacesAndNewlines)

        if(taskname.isEmpty){
            textFieldName.layer.borderColor = UIColor.red.cgColor
            return
        }

        do {
            try db.run(tasks.insert(name <- taskname, done <- 0))
            if db.changes > 0 {
                print("Task added successfully")
            } else {
                print("failed inserting task")
                return
            }
        } catch let error {
            print("failed inserting task: \(error)")
            return
        }

        textFieldName.text = ""

        updateList()
    }

    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return taskList.count
    }

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = UITableViewCell(style: UITableViewCellStyle.default, reuseIdentifier: "cell")
        let task: Task
        task = taskList[indexPath.row]
        cell.textLabel?.text = task.name
        return cell
    }

    func updateList(){

        taskList.removeAll()

        do {
            for task in try db.prepare(tasks) {
                taskList.append(Task(id: task[id], name: task[name], done: task[done]))
            }
        } catch let error {
            print("error preparing statement: \(error)")
            return
        }

        self.tableViewTasks.reloadData()

    }
    
    func startDbAccess(){

        print("starting db access!")

        do {
            try db.run(tasks.create(ifNotExists: true) { t in
                t.column(id, primaryKey: true)
                t.column(name)
                t.column(done)
            })
            //try db.execute("CREATE TABLE IF NOT EXISTS Tasks (id INTEGER PRIMARY KEY, name TEXT, done INTEGER)")
        } catch let error {
            print("error creating table: \(error)")
        }
        
        updateList()

        db.onSync {
          print("------- onSync -------")
          self.updateList()
        }

    }

    override func viewDidLoad() {
        super.viewDidLoad()

        let fileURL = try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: false)
            .appendingPathComponent("AppDatabase.sqlite")
        
        //let uri = "file:" + fileURL.path + "?node=primary&bind=tcp://0.0.0.0:1234"
        let uri = "file:" + fileURL.path + "?node=secondary&connect=tcp://192.168.1.45:1234"

        do {
            db = try Connection(uri)
        } catch let error {
            print("error opening the database: \(error)")
            return
        }

        print("the database is open")

        if db.isReady() {
          // the user is already logged in
          startDbAccess()
        } else {
          // wait until the db is ready before making any access
          db.onReady {
            print("------- onReady -------")
            self.startDbAccess()
          }
        }

    }

}
